@@ -38,6 +38,7 @@ class deML (list):
 		self._deML_summary_filename = deML_summary_filename
 		self._pipeline_log_filename = pipeline_log_filename
 
+		# Assign the output options
 		self._keep_unknown = keep_unknown
 		self._keep_failed = keep_failed
 		self._keep_indices = keep_indices
@@ -98,8 +99,8 @@ class deML (list):
 		2) Store the files within a sub-directory
 		''' 
 		_processOptionalOutput('*_unknown_*.fq.gz', 'Unknown' if self._keep_unknown else None)
-		_processOptionalOutput('*.fail.fq.gz', 'Failed' if self._keep_unknown else None)
-		_processOptionalOutput('*_i[1-2].fq.gz', 'Indices' if self._keep_unknown else None)
+		_processOptionalOutput('*.fail.fq.gz', 'Failed' if self._keep_failed else None)
+		_processOptionalOutput('*_i[1-2].fq.gz', 'Indices' if self._keep_indices else None)
 
 		# Rename the R1/2 demultiplexed reads
 		for deML_filename in os.listdir(out_dir):
@@ -107,12 +108,25 @@ class deML (list):
 			if not os.path.isfile(deML_file): continue
 			os.rename(deML_file, re.sub(r'tmp_(.*)r([1-2].fq.gz)', r'\1R\2', deML_file))
 
-		# Append and remove the deML log
-		self._appendLog()
+		# Log the contents of the summary file
+		self._cleanSummary()
+		self._logSummary()
 
 		logging.info('Finished FASTQ paired-index demultiplex')
 
-	def _appendLog (self):
+	def _cleanSummary (self):
+
+		# Read in the orginal contents of the summary file
+		with open(self._deML_summary_filename, 'r') as summary_file:
+			summary_lines = summary_file.readlines()
+
+		# Keep the orginal contents and remove the dash lines
+		with open(self._deML_summary_filename, 'w') as summary_file:
+			for summary_line in summary_lines:
+				if summary_line.strip() == '-' * 32: continue
+				summary_file.write(summary_line)
+
+	def _logSummary (self):
 
 		# Assign the log lines
 		start_message = ('#' * 12) + ' Start deML Summary ' + ('#' * 12)
@@ -121,16 +135,18 @@ class deML (list):
 
 		# Append the log file
 		deML_log = open(self._deML_summary_filename, 'r').read()
-		logging.info(f'\n{spacer_line}\n{start_message}\n{deML_log}{end_massage}\n{spacer_line}\n')
+		logging.info(f'\n{spacer_line}\n{start_message}\n{deML_log}{end_massage}\n{spacer_line}')
 
 	def _processIndex (self):
 
 		def _updateExcelHeader (excel_dataframe):
 
+			# Create arguments to store header column names
 			well_col = None
 			i7_col = None
 			i5_col = None
 
+			# Assign the header column names
 			for index_col in excel_dataframe.columns:
 				if 'sample' in str(index_col).lower() or 'well' in str(index_col).lower():
 					if well_col: raise Exception(f'Well column assignment error: {self.index_format}')
@@ -142,14 +158,24 @@ class deML (list):
 					if i5_col: raise Exception(f'i5 column assignment error: {self.index_format}')
 					i5_col = index_col
 
-			if not well_col or not i5_col or not i5_col: raise Exception(f'Unable to update excel header')
+			# Confirm all the headers were found
+			if not well_col or not i7_col or not i5_col: raise Exception(f'Unable to update header')
 
-			return excel_dataframe.rename({i7_col: '#Index1', i5_col: 'Index2', well_col: 'Name'}, axis = 'columns')
+			# Return the updated header names
+			return excel_dataframe.rename({i7_col: self._index_i7_col, i5_col: self._index_i5_col, well_col: self._index_well_col}, axis = 'columns')
 
-		def _hasWhitespace ():
-			if index_dataframe[self._index_i7_col].str.contains('\s+').any(): return True
-			elif index_dataframe[self._index_i5_col].str.contains('\s+').any(): return True
-			else: return False
+		def _hasWhitespace (cols):
+
+			if not isinstance(cols, list): cols = [cols]
+			for col in cols:
+				if index_dataframe[col].str.contains('\s+').any(): return True
+			return False
+
+		def _removeWhitespace (cols):
+
+			if not isinstance(cols, list): cols = [cols]
+			for col in cols:
+				index_dataframe[col] = index_dataframe[col].str.replace(' ', '')
 
 		def _revCompBarcodes (barcode):
 
@@ -161,10 +187,7 @@ class deML (list):
 		if self.index_format == 'excel':
 			index_dataframe = pd.read_excel(self.index, sheet_name = self.excel_sheet, engine = 'openpyxl')
 			index_dataframe = _updateExcelHeader(index_dataframe)
-		elif self.index_format == 'tsv': 
-			index_dataframe = pd.read_csv(self.index, sep = '\t')
-			index_dataframe = _updateExcelHeader(index_dataframe)
-
+		elif self.index_format == 'tsv': index_dataframe = pd.read_csv(self.index, sep = '\t')
 		else: raise Exception(f'Unknown index format: {self.index_format}')
 
 		# Check for empty rows
@@ -179,14 +202,13 @@ class deML (list):
 		if number_of_columns < len(self._index_cols): raise Exception(f'Expected {len(self._index_cols)} columns, found {number_of_columns} columns')
 		elif number_of_columns > len(self._index_cols): index_dataframe = index_dataframe[self._index_cols]
 
-		# NEED TO ADD SAMPLE FILTER
-		index_dataframe[self._index_well_col] = index_dataframe[self._index_well_col].str.strip()
-		index_dataframe[self._index_well_col] = index_dataframe[self._index_well_col].str.replace(' ', '_')
+		# Remove any whitespace is the wells/samples, if needed
+		sample_whitespace = _hasWhitespace(self._index_well_col)
+		if sample_whitespace: _removeWhitespace(self._index_well_col)
 
-		# Remove any whitespace in barcodes, if needed
-		barcode_whitespace = _hasWhitespace()
-		if barcode_whitespace: index_dataframe[self._index_i5_col] = index_dataframe[self._index_i5_col].str.replace(' ', '')
-		if barcode_whitespace: index_dataframe[self._index_i7_col] = index_dataframe[self._index_i7_col].str.replace(' ', '')
+		# Remove any whitespace in the barcodes, if needed
+		barcode_whitespace = _hasWhitespace(self._barcode_cols)
+		if barcode_whitespace: _removeWhitespace(self._barcode_cols)
 
 		# Check the length of the barcodes - whitespace must be removed prior
 		if not (index_dataframe[self._index_i7_col].str.len() == self._index_barcode_len).all():
@@ -199,7 +221,7 @@ class deML (list):
 		if self.i5_reverse_complement: index_dataframe[self._index_i5_col] = index_dataframe[self._index_i5_col].apply(_revCompBarcodes)
 
 		# Check if the file needs to be formatted
-		#if not empty_rows and not self.i7_reverse_complement and not self.i5_reverse_complement and not barcode_whitespace and number_of_columns == len(self._index_cols): return
+		if self.index_format == 'excel' and not empty_rows and not self.i7_reverse_complement and not self.i5_reverse_complement and not sample_whitespace and not barcode_whitespace and number_of_columns == len(self._index_cols): return
 		
 		# Reorder and rename the columns
 		index_dataframe = index_dataframe[self._index_cols]
@@ -208,7 +230,7 @@ class deML (list):
 		self.index = f'{self.index}.deML.formatted'
 		index_dataframe.to_csv(self.index , sep = '\t', index = False)
 
-		logging.info(f'Index file ({self.index}) processed and assigned.')
+		logging.info(f'Index file ({self.index}) processed and assigned')
 
 	def _call (self):
 		'''
@@ -222,10 +244,10 @@ class deML (list):
 				If deML stderr returns an error
 		'''
 
-		# plink subprocess call
+		# deML subprocess call
 		deML_call = subprocess.Popen(self.deML_arg_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
-		# Wait for plink to finish
+		# Wait for deML to finish
 		deML_out, deML_err = deML_call.communicate()
 
 		# If needed, convert bytes to string
@@ -244,7 +266,7 @@ class deML (list):
 		spacer_line = '#' * 44
 
 		# Append the log file
-		logging.info(f'\n{spacer_line}\n{start_message}\n{deML_err}{end_massage}\n{spacer_line}\n')
+		logging.info(f'\n{spacer_line}\n{start_message}\n{deML_err}{end_massage}\n{spacer_line}')
 
 	@staticmethod
 	def _check_for_errors (deML_stderr):
